/* Mon Jun 22 11:49:32 2026 */
/* M  ChangeLog */
/* M  doc/DBD-File.3 */
/* M  doc/DBD-File.man */
/* M  doc/DBI.3 */
/* M  doc/DBI.html */
/* M  doc/DBI.man */
/* M  doc/DBI.md */
/*  M lib/DBI/Changes.pm */
#define DBIXS_RELEASE  1
#define DBIXS_VERSION  649
#define DBIXS_REVISION 1748
